/*
 * charmap.c
 */

#include "init.h"
#include "charmap.h"
#include "face.h"
#include "error.h"

static void
tt_free_charmap(charmap)
	TT_CharMap *charmap;
{
	free(charmap);
}

static VALUE
tt_face_get_charmap_id(self, index)
	VALUE self, index;
{
	TT_UShort platform, encoding;
	TT_Face *face;
	TT_Error error;

	Data_Get_Struct(self, TT_Face, face);
	error = TT_Get_CharMap_ID(*face, FIX2INT(index), &platform, &encoding);
	if (error) tt_error_raise(error);

	return rb_assoc_new(INT2FIX(platform), INT2FIX(encoding));
}

static VALUE
tt_face_get_charmap(self, index)
	VALUE self, index;
{
	TT_CharMap *charmap;
	TT_Face *face;
	int i;
	VALUE obj;
	TT_Error error;

	Data_Get_Struct(self, TT_Face, face);
	obj = Data_Make_Struct(cTT_charmap, TT_CharMap, 0, tt_free_charmap, charmap);
	i = FIX2INT(index);
	error = TT_Get_CharMap(*face, i, charmap);
	if (error) tt_error_raise(error);

	return obj;
}

static VALUE
tt_charmap_aref(self, code)
{
	TT_CharMap *charmap;
	int c;
	TT_UShort index;

	Data_Get_Struct(self, TT_CharMap, charmap);
	c = FIX2INT(code);
	index = TT_Char_Index(*charmap, c);

	return INT2FIX(index);
}

void
tt_init_charmap()
{
	/*
	 * character mapping support
	 */
	cTT_charmap = rb_define_class_under(mTT, "Charmap", rb_cObject);

	rb_define_method(cTT_face, "charmap_id", tt_face_get_charmap_id, 1);
	rb_define_method(cTT_face, "charmap", tt_face_get_charmap, 1);
	rb_define_method(cTT_charmap, "[]", tt_charmap_aref, 1);
}
