/*
 * face.c
 */

#include "init.h"
#include "face.h"
#include "faceprop.h"
#include "error.h"
#include "debug.h"

static void
tt_free_face(face)
	TT_Face *face;
{
	Debug("free face begin");
	/* if not close */
	if (inited && face->z) {
		TT_Error error;

		error = TT_Close_Face(*face);
		if (error) tt_error_raise(error);
		face->z = NULL; /* for close again */
	}
	Debug("free face end");
}

static VALUE
tt_face_open(self, path_name)
	VALUE self, path_name;
{
	TT_Face *face;
	VALUE obj;
	TT_Error error;

	obj = Data_Make_Struct(self, TT_Face, 0, tt_free_face, face);
	error = TT_Open_Face(tt_engine, STR2CSTR(path_name), face);
	if (error) tt_error_raise(error);

	return obj;
}

static VALUE
tt_face_open_collection(self, path_name, index)
	VALUE self, path_name, index;
{
	TT_Face *face = NULL;
	VALUE obj;
	TT_ULong i;
	TT_Error error;

	obj = Data_Make_Struct(self, TT_Face, 0, tt_free_face, face);
	i = NUM2INT(index);
	error = TT_Open_Collection(tt_engine, STR2CSTR(path_name), i, face);
	if (error) tt_error_raise(error);

	return obj;
}

static VALUE
tt_face_get_properties(self)
	VALUE self;
{
	TT_Face *face;
	TT_Face_Properties faceprop;
	TT_Error error;

	Data_Get_Struct(self, TT_Face, face);
	error = TT_Get_Face_Properties(*face, &faceprop);
	if (error) tt_error_raise(error);

	return tt_faceprop_setup(&faceprop);
}

static VALUE
tt_face_flush(self)
	VALUE self;
{
	TT_Face *face;
	TT_Error error;

	Data_Get_Struct(self, TT_Face, face);
	error = TT_Flush_Face(*face);
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_face_close(self)
	VALUE self;
{
	TT_Face *face;
	TT_Error error;

	Debug("face close begin");
	Data_Get_Struct(self, TT_Face, face);
	error = TT_Close_Face(*face);
	if (error) tt_error_raise(error);
	face->z = NULL; /* for close again */
	Debug("face close end");

	return Qnil;
}

void
tt_init_face()
{
	/*
	 * face management
	 */
	cTT_face = rb_define_class_under(mTT, "Face", rb_cObject);

	rb_define_singleton_method(cTT_face, "open", tt_face_open, 1);
	rb_define_singleton_method(cTT_face, "open_collection", tt_face_open_collection, 2);
	rb_define_method(cTT_face, "properties", tt_face_get_properties, 0);
	rb_define_method(cTT_face, "flush", tt_face_flush, 0);
	rb_define_method(cTT_face, "close", tt_face_close, 0);
}
