/*
 * init.c
 */

#include "init.h"
#include "bbox.h"
#include "face.h"
#include "faceprop.h"
#include "scale.h"
#include "smetrics.h"
#include "outline.h"
#include "glyph.h"
#include "gmetrics.h"
#include "raster.h"
#include "matrix.h"
#include "charmap.h"
#include "names.h"
#include "error.h"
#include "debug.h"

static VALUE
tt_version(self)
	VALUE self;
{
	int major, minor;
	TT_Error error;
	error = TT_FreeType_Version(&major, &minor);
	if (error) tt_error_raise(error);
	return rb_assoc_new(INT2NUM(major), INT2NUM(minor));
}

static void
tt_init()
{
	if (!inited) {
		TT_Error error;

		Debug("init begin");
		error = TT_Init_FreeType(&tt_engine);
		if (error) tt_error_raise(error);
		inited++;
		Debug("init end");
	}
}

static void
tt_finalize()
{
	TT_Error error;

	Debug("finalize begin");
	inited = 0;
	error = TT_Done_FreeType(tt_engine);
	if (error) tt_error_raise(error);
	Debug("finalize end");
}

void
Init_freetype()
{
	mTT = rb_define_module("FreeType");

	rb_define_module_function(mTT, "version", tt_version, 0);

	tt_init();
	tt_init_bbox();
	tt_init_face();
	tt_init_faceprop();
	tt_init_scale();
	tt_init_smetrics();
	tt_init_outline();
	tt_init_glyph();
	tt_init_gmetrics();
	tt_init_raster();
	tt_init_matrix();
	tt_init_charmap();
	tt_init_names();
	tt_init_error();

	/* finalize */
	rb_set_end_proc(tt_finalize, 0);
}
