/*
 * matrix.c
 */

#include <math.h>

#include "init.h"
#include "attr.h"
#include "matrix.h"

#define  PI  3.1415926535

static void
tt_free_matrix(matrix)
	TT_Matrix *matrix;
{
	free(matrix);
}

static VALUE
tt_matrix_new(self)
	VALUE self;
{
	TT_Matrix *matrix;
	VALUE obj;
	obj = Data_Make_Struct(self, TT_Matrix, 0, tt_free_matrix, matrix);
	return obj;
}

DefineNumericAttr(Matrix, xx)
DefineNumericAttr(Matrix, xy)
DefineNumericAttr(Matrix, yx)
DefineNumericAttr(Matrix, yy)

static VALUE
tt_matrix_rotate(self, angle)
	VALUE self, angle;
{
	TT_Matrix *matrix;
	VALUE obj;
	float rad;

	obj = Data_Make_Struct(self, TT_Matrix, 0, tt_free_matrix, matrix);
	rad = NUM2INT(angle) * PI / 512;
	matrix->xx = (TT_Fixed)(cos( rad ) * (1L<<16));
	matrix->xy = (TT_Fixed)(sin( rad ) * (1L<<16));
	matrix->yx = -matrix->xy;
	matrix->yy = +matrix->xx;

	return obj;
}

static VALUE
tt_matrix_zoom(self, factor)
	VALUE self, factor;
{
	TT_Matrix *matrix;
	VALUE obj;

	obj = Data_Make_Struct(self, TT_Matrix, 0, tt_free_matrix, matrix);
	matrix->xx = matrix->yy = NUM2INT(factor) * (1L<<16);
	matrix->xy = matrix->yx = 0;

	return obj;
}

void
tt_init_matrix()
{
	cTT_Matrix = rb_define_class_under(mTT, "Matrix", rb_cObject);
	rb_define_singleton_method(cTT_Matrix, "new", tt_matrix_new, 0);
	SetupAttr(Matrix, xx)
	SetupAttr(Matrix, xy)
	SetupAttr(Matrix, yx)
	SetupAttr(Matrix, yy)
	rb_define_singleton_method(cTT_Matrix, "rotate", tt_matrix_rotate, 1);
	rb_define_singleton_method(cTT_Matrix, "zoom", tt_matrix_zoom, 1);
}
