/*
 * outline.c
 */

#include "init.h"
#include "outline.h"
#include "error.h"
#include "debug.h"

static void
tt_free_outline(outline)
	TT_Outline *outline;
{
	/*
	 * true  : Glyph.outline
	 * false : Outline.new
	 */
	if (!outline->owner) {
		TT_Error error;

		Debug("free outline begin");
		error = TT_Done_Outline(outline);
		if (error) tt_error_raise(error);
		Debug("free outline end");
	}
}

static VALUE
tt_outline_new(self, points, contours)
	VALUE self, points, contours;
{
	TT_Outline *outline;
	VALUE obj;
	TT_Error error;

	/*
	 * create user outline.
	 * user outlines are not destroyed by finalizer.
	 */
	obj = Data_Make_Struct(self, TT_Outline, 0, tt_free_outline, outline);
	error = TT_New_Outline(FIX2INT(points), FIX2INT(contours), outline);
	if (error) tt_error_raise(error);

	return obj;
}

static VALUE
tt_outline_done(self)
	VALUE self;
{
	TT_Outline *outline;
	TT_Error error;

	Data_Get_Struct(self, TT_Outline, outline);
	error = TT_Done_Outline(outline);
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_outline_copy(self, target_)
	VALUE self, target_;
{
	TT_Outline *source, *target;
	TT_Error error;

	Data_Get_Struct(self, TT_Outline, source);
	Data_Get_Struct(target_, TT_Outline, target);
	error = TT_Copy_Outline(source, target);
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_outline_translate(self, x, y)
{
	TT_Outline *outline;

	Data_Get_Struct(self, TT_Outline, outline);
	TT_Translate_Outline(outline, FIX2INT(x), FIX2INT(y));

	return Qnil;
}

static VALUE
tt_outline_transform(self, matrix_)
{
	TT_Outline *outline;
	TT_Matrix  *matrix;

	Data_Get_Struct(self, TT_Outline, outline);
	Data_Get_Struct(matrix_, TT_Matrix , matrix);
	TT_Transform_Outline(outline, matrix);

	return Qnil;
}

static VALUE
tt_outline_get_bitmap(self, map_)
	VALUE self, map_;
{
	TT_Outline *outline;
	TT_Raster_Map *map;

	Data_Get_Struct(self, TT_Outline, outline);
	Data_Get_Struct(map_, TT_Raster_Map , map);
	if (!map->size) {
		map->size = map->rows * map->cols;
		map->bitmap = ALLOC_N(char, map->size);
	}
	TT_Get_Outline_Bitmap(tt_engine, outline, map);

	return Qnil;
}

static VALUE
tt_outline_get_pixmap(self, map_)
	VALUE self, map_;
{
	TT_Outline *outline;
	TT_Raster_Map *map;

	Data_Get_Struct(self, TT_Outline, outline);
	Data_Get_Struct(map_, TT_Raster_Map , map);
	if (!map->size) {
		Debug("outline new pixmap");
		map->size = map->rows * map->cols;
		map->bitmap = ALLOC_N(char, map->size);
	}
	TT_Get_Outline_Pixmap(tt_engine, outline, map);

	return Qnil;
}

void
tt_init_outline()
{
	/*
	 * outline support
	 */
	cTT_outline = rb_define_class_under(mTT, "Outline", rb_cObject);

	rb_define_singleton_method(cTT_outline, "new", tt_outline_new, 2);
	rb_define_method(cTT_outline, "done", tt_outline_done, 0);
	rb_define_method(cTT_outline, "copy", tt_outline_copy, 1);
	rb_define_method(cTT_outline, "transform", tt_outline_transform, 1);
	rb_define_method(cTT_outline, "translate", tt_outline_translate, 2);
	rb_define_method(cTT_outline, "get_bitmap", tt_outline_get_bitmap, 1);
	rb_define_method(cTT_outline, "get_pixmap", tt_outline_get_pixmap, 1);
}
