/*
 * raster.c
 */

#include "init.h"
#include "attr.h"
#include "raster.h"
#include "error.h"

static void
tt_free_raster(raster)
	TT_Raster_Map *raster;
{
	free(raster->bitmap);
	free(raster);
}

static VALUE
tt_raster_set_gray_palette(self, ary)
	VALUE self, ary;
{
	TT_Byte palette[5];
	TT_Error error;

	if (RARRAY(ary)->len != 5) {
		rb_raise(rb_eArgError, "gray palette length must be 5");
	}
	palette[0] = FIX2INT(RARRAY(ary)->ptr[0]); /* background(white) */
	palette[1] = FIX2INT(RARRAY(ary)->ptr[1]); /* light             */
	palette[2] = FIX2INT(RARRAY(ary)->ptr[2]); /* medium            */
	palette[3] = FIX2INT(RARRAY(ary)->ptr[3]); /* dark              */
	palette[4] = FIX2INT(RARRAY(ary)->ptr[4]); /* foreground(black) */

	error = TT_Set_Raster_Gray_Palette(tt_engine, palette);
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_raster_new(self)
	VALUE self;
{
	TT_Raster_Map *raster;
	VALUE obj;

	obj = Data_Make_Struct(self, TT_Raster_Map, 0, tt_free_raster, raster);
	raster->rows = 0;
	raster->cols = 0;
	raster->width = 0;
	raster->flow = TT_Flow_Down;
	raster->bitmap = ALLOC(char);
	raster->size = 0;

	return obj;
}

DefineNumericAttr(Raster_Map, rows)
DefineNumericAttr(Raster_Map, cols)
DefineNumericAttr(Raster_Map, width)
DefineNumericAttr(Raster_Map, flow)

static VALUE
tt_raster_fill(self, value)
	VALUE self, value;
{
	TT_Raster_Map *raster;
	int v;

	Data_Get_Struct(self, TT_Raster_Map, raster);
	v = NUM2INT(value);
	memset(raster->bitmap, v, raster->size);

	return Qnil;
}

static VALUE
tt_raster_aref(self, index)
	VALUE self, index;
{
	TT_Raster_Map *raster;
	int i;
	char *map;
	VALUE value;

	Data_Get_Struct(self, TT_Raster_Map, raster);
	i = NUM2INT(index);
	if (i < 0 || raster->size <=i) {
		rb_raise(rb_eIndexError, "index out of range");
	}
	map = (char *)(raster->bitmap);
	value = INT2NUM(map[i]);

	return value;
}

static VALUE
tt_raster_each(self)
	VALUE self;
{
	TT_Raster_Map *raster;
	int i;
	char *map;
	VALUE value;

	Data_Get_Struct(self, TT_Raster_Map, raster);
	for (i=0; i<raster->size; i++) {
		map = (char *)(raster->bitmap);
		value = INT2NUM(map[i]);
		rb_yield(value);
	}
	return self;
}

static VALUE
tt_raster_getc(self, col, row)
	VALUE self, col, row;
{
	TT_Raster_Map *raster;
	int x, y;
	char *map;
	VALUE value;

	Data_Get_Struct(self, TT_Raster_Map, raster);
	x = NUM2INT(col);
	y = NUM2INT(row);
	if (x < 0 || raster->cols <=x) {
		rb_raise(rb_eIndexError, "col out of range");
	}
	if (y < 0 || raster->rows <=y) {
		rb_raise(rb_eIndexError, "row out of range");
	}
	map = (char *)(raster->bitmap);
	value = INT2NUM(map[x+y*raster->cols]);

	return value;
}

void
tt_init_raster()
{
	cTT_Raster_Map = rb_define_class_under(mTT, "Raster", rb_cObject);
	rb_define_module_function(cTT_Raster_Map, "gray_palette=", tt_raster_set_gray_palette, 1);
	rb_define_singleton_method(cTT_Raster_Map, "new", tt_raster_new, 0);
	rb_define_method(cTT_Raster_Map, "fill", tt_raster_fill, 1);
	rb_define_method(cTT_Raster_Map, "[]", tt_raster_aref, 1);
	rb_define_method(cTT_Raster_Map, "each", tt_raster_each, 0);
	rb_define_method(cTT_Raster_Map, "getc", tt_raster_getc, 2);

	SetupAttr(Raster_Map, rows)
	SetupAttr(Raster_Map, cols)
	SetupAttr(Raster_Map, width)
	SetupAttr(Raster_Map, flow)

	rb_define_const(cTT_Raster_Map, "FLOW_DOWN", TT_Flow_Down);
	rb_define_const(cTT_Raster_Map, "FLOW_UP"  , TT_Flow_Up  );
}
