#!/usr/local/bin/ruby

require 'xlib'
require 'freetype'
require 'kconv'

DIS = Xlib::Display.new
WIN = DIS.root.new_window 320, 200
WIN.title = $0
WIN.show
GC = WIN.new_gc

major, minor = FreeType.version
print "FreeType version #{major}.#{minor}\n"

true_type = '/usr/X11R6/lib/X11/fonts/xtt/watanabe-mincho.ttf'
face = FreeType::Face.open true_type

prop = face.properties

for i in 0...prop.num_charmaps
  platform, encoding = face.charmap_id(i)
  p [platform, encoding]
end

cmap_id = 1
cmap = face.charmap(cmap_id)

@glyph = FreeType::Glyph.new face

@scale = FreeType::Scale.new face
@scale.point_size = 36
p @scale.metrics

@rot_m = FreeType::Matrix.rotate -60
@zoo_m = FreeType::Matrix.zoom 3

FreeType::Raster.gray_palette = [0,1,2,3,4]
@palette = [
  0x000000,
  0x880000,
  0xaa0000,
  0xcc0000,
  0xff0000,
].collect { |i| DIS.alloc_color Xlib::Color.new(i) }

def map_new
  @map = FreeType::Raster.new
  @map.rows = WIN.height
  @map.width = WIN.width
  @map.cols = @map.width
  @map.flow = FreeType::Raster::FLOW_UP
end

map_new
@posx, @posy = 0, 0

def mapping
  o = @glyph.outline
  o.translate @posx, @posy
  o.translate 18*64, -5*64
  o.transform @rot_m
  o.transform @zoo_m
  o.get_pixmap @map
end

def draw(num)
  FreeType.load_glyph @scale, @glyph, num
  print "num #{num}\n"
  mapping

  for row in 0...@map.rows
    for col in 0...@map.cols
      i = @map.getc col, row 
      if i != 0
      # print "#{col} #{row} #{i}\n"
        GC.fg = @palette[i]
        GC.draw_point col, row
      end
    end
  end

  metrics = @glyph.metrics
p metrics
  @posx+= metrics.advance
end

euc = "ȶ"

euc.scan(/(.)/) { |c,|
  u = Kconv.tosjis(c)
  char = case u.size
  when 1; u[1]
  when 2; u[1]+(u[0]<<8)
  end
  draw cmap[char]
  printf "char: %04x\n", char
}

print "done\n"
DIS.flush
WIN.event 'button_press' do exit end

while DIS.event
end
