/*
 * scale.c
 */

#include "init.h"
#include "face.h"
#include "scale.h"
#include "smetrics.h"
#include "error.h"
#include "debug.h"

static void
tt_free_scale(scale)
	TT_Instance *scale;
{
	Debug("free scale begin");
	/* if not done */
	if (inited && scale->z) {
		TT_Error error;

		error = TT_Done_Instance(*scale);
		if (error) tt_error_raise(error);
		scale->z = NULL; /* for done again */
	}
	Debug("free scale end");
}

static VALUE
tt_scale_new(self, face_obj)
	VALUE self, face_obj;
{
	TT_Instance *scale;
	TT_Face *face;
	VALUE obj;
	TT_Error error;

	Data_Get_Struct(face_obj, TT_Face, face);
	obj = Data_Make_Struct(self, TT_Instance, 0, tt_free_scale, scale);
	error = TT_New_Instance(*face, scale);
	if (error) tt_error_raise(error);

	return obj;
}

static VALUE
tt_scale_done(self)
	VALUE self;
{
	TT_Instance *scale;
	TT_Error error;

	Data_Get_Struct(self, TT_Instance, scale);
	error = TT_Done_Instance(*scale);
	if (error) tt_error_raise(error);
	scale->z = NULL; /* for done again */

	return Qnil;
}

static VALUE
tt_scale_set_resolution(self, resolution)
	VALUE self, resolution;
{
	TT_Instance *scale;
	TT_Error error;
	TT_UShort res;

	Data_Get_Struct(self, TT_Instance, scale);
	res = NUM2INT(resolution);
	error = TT_Set_Instance_Resolutions(*scale, res, res);
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_scale_set_point_size(self, ptsize)
	VALUE self, ptsize;
{
	TT_Instance *scale;
	TT_Error error;

	Data_Get_Struct(self, TT_Instance, scale);
	error = TT_Set_Instance_PointSize(*scale, NUM2INT(ptsize));
	if (error) tt_error_raise(error);

	return Qnil;
}

static VALUE
tt_scale_get_metrics(self)
	VALUE self;
{
	TT_Instance *scale;
	TT_Instance_Metrics smetrics;
	TT_Error error;

	Data_Get_Struct(self, TT_Instance, scale);
	error = TT_Get_Instance_Metrics(*scale, &smetrics);
	if (error) tt_error_raise(error);

	return tt_smetrics_setup(&smetrics);
}

void
tt_init_scale()
{
	/*
	 * scale management
	 */
	cTT_Scale = rb_define_class_under(mTT, "Scale", rb_cObject);

	rb_define_singleton_method(cTT_Scale, "new", tt_scale_new, 1);
	rb_define_method(cTT_Scale, "done", tt_scale_done, 0);
	rb_define_method(cTT_Scale, "resolution=", tt_scale_set_resolution, 1);
	rb_define_method(cTT_Scale, "point_size=", tt_scale_set_point_size, 1);
	rb_define_method(cTT_Scale, "metrics", tt_scale_get_metrics, 0);
}
